﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Text;

namespace $rootnamespace$.HojasCalculo.Excel
{
    internal class ComWrapper : StandardOleMarshalObject, IDisposable
    {
        private object comObject;

        public ComWrapper(object comObject)
        {
            this.comObject = comObject;
        }

        public object ComObject
        {
            get { return comObject; }
        }

        protected internal object GetProperty(string name)
        {
            return comObject.GetType().InvokeMember(name, BindingFlags.GetProperty | BindingFlags.Public, null, comObject, null);
        }

        protected internal object GetProperty(string name, params object[] paramArray)
        {
            return comObject.GetType().InvokeMember(name, BindingFlags.GetProperty | BindingFlags.Public, null, comObject, paramArray);
        }

        protected internal void SetProperty(string name, object value)
        {
            comObject.GetType().InvokeMember(name, BindingFlags.SetProperty | BindingFlags.Public, null, comObject, new object[] { value });
        }

        protected internal void SetProperty(string name, params object[] paramArray)
        {
            comObject.GetType().InvokeMember(name, BindingFlags.SetProperty | BindingFlags.Public, null, comObject, paramArray);
        }

        protected internal void SetProperty(string name, ComWrapper wrapper)
        {
            SetProperty(name, wrapper.comObject);
        }

        protected internal object InvokeMethod(string name)
        {
            return comObject.GetType().InvokeMember(name, BindingFlags.InvokeMethod | BindingFlags.Public, null, comObject, null);
        }

        protected internal object InvokeMethod(string name, params object[] paramArray)
        {
            return comObject.GetType().InvokeMember(name, BindingFlags.InvokeMethod | BindingFlags.Public, null, comObject, paramArray);
        }

        public void Dispose()
        {
            var iUnknown = Marshal.GetIUnknownForObject(comObject);
            Marshal.Release(iUnknown);
        }
    }

    internal interface IIndexedProperty<TValue, TKey>
    {
        TValue this[TKey Key] { get; set; }
    }

    internal class IndexedProperty<TValue, TKey> : IIndexedProperty<TValue, TKey>
    {
        private readonly ComWrapper owner;
        private readonly string propertyName;

        public IndexedProperty(ComWrapper owner, string propertyName)
        {
            this.owner = owner;
            this.propertyName = propertyName;
        }

        public TValue this[TKey Key]
        {
            get { return (TValue)owner.GetProperty(propertyName, Key); }
            set { owner.SetProperty(propertyName, Key, (TValue)value); }
        }
    }

    internal class ExcelApplication : ComWrapper
    {
        public static bool EstaInstalado
        {
            get { return Type.GetTypeFromProgID("Excel.Application") != null; }
        }

        public ExcelCalculation DeferredCalculation()
        {
            return new ExcelCalculation(this, XlCalculation.xlManual);
        }

        public ExcelApplication() : base(Activator.CreateInstance(Type.GetTypeFromProgID("Excel.Application"))) { }

        public Workbooks Workbooks
        {
            get { return new Workbooks(GetProperty("Workbooks")); }
        }

        public bool Visible
        {
            get { return (bool)GetProperty("Visible"); }
            set { SetProperty("Visible", value); }
        }

        public Window ActiveWindow
        {
            get { return new Window(GetProperty("ActiveWindow")); }
        }

        public bool ScreenUpdating
        {
            get { return (bool)GetProperty("ScreenUpdating"); }
            set { SetProperty("ScreenUpdating", value); }
        }

        public XlCalculation Calculation
        {
            get { return (XlCalculation)GetProperty("Calculation"); }
            set { SetProperty("Calculation", value); }
        }

        public bool DisplayAlerts
        {
            get { return (bool)GetProperty("DisplayAlerts"); }
            set { SetProperty("DisplayAlerts", value); }
        }

        public void Calculate()
        {
            InvokeMethod("Calculate");
        }

        public void Quit()
        {
            InvokeMethod("Quit");
        }

        public Range Intersect(Range arg1, Range arg2)
        {
            var result = InvokeMethod("Intersect", arg1.ComObject, arg2.ComObject);
            if (result == null)
                return null;
            return new Range(result);
        }

        public double InchesToPoints(double inches)
        {
            return (double)InvokeMethod("InchesToPoints", inches);
        }

        public int SheetsInNewWorkbook
        {
            get
            {
                var r = GetProperty("SheetsInNewWorkbook");
                return Convert.ToInt32(GetProperty("SheetsInNewWorkbook"));
            }
            set { SetProperty("SheetsInNewWorkbook", value); }
        }

        public static int Rgb(int r, int g, int b)
        {
            return (b * 256 * 256) + (g * 256) + r;
        }
    }

    internal class Workbooks : ComWrapper
    {
        internal Workbooks(object workbooks) : base(workbooks) { }

        public Workbook this[int index]
        {
            get { return new Workbook(GetProperty("Item", index)); }
        }

        public Workbook Add()
        {
            return new Workbook(InvokeMethod("Add"));
        }

        public Workbook Open(string fileName)
        {
            return new Workbook(InvokeMethod("Open", fileName));
        }

        public Workbook Open(string fileName, XlUpdateLinks updateLinks)
        {
            return new Workbook(InvokeMethod("Open", fileName, updateLinks));
        }

        public Workbook Open(string fileName, XlUpdateLinks updateLinks, XlFileAccess readOnly)
        {
            return new Workbook(InvokeMethod("Open", fileName, updateLinks, readOnly));
        }
    }

    internal class Workbook : ComWrapper
    {
        internal Workbook(object workbook) : base(workbook) { }

        public Worksheets Worksheets
        {
            get { return new Worksheets(GetProperty("Worksheets")); }
        }

        public void SaveAs(string filename)
        {
            InvokeMethod("SaveAs", filename);
        }

        public void Close()
        {
            InvokeMethod("Close");
        }

        public void Close(bool saveChanges)
        {
            InvokeMethod("Close", saveChanges);
        }

        public string FullName
        {
            get { return (string)GetProperty("FullName"); }
        }

        public IIndexedProperty<object, object> Colors
        {
            get { return new IndexedProperty<object, object>(this, "Colors"); }
        }
    }

    internal class Worksheets : ComWrapper, IEnumerable<Worksheet>
    {
        internal Worksheets(object worksheets) : base(worksheets) { }

        public Worksheet this[int index]
        {
            get { return new Worksheet(GetProperty("Item", index)); }
        }

        public Worksheet this[string name]
        {
            get { return new Worksheet(GetProperty("Item", name)); }
        }

        public int Count
        {
            get { return (int)GetProperty("Count"); }
        }

        public Worksheet Add()
        {
            return new Worksheet(InvokeMethod("Add"));
        }

        public string Name
        {
            get { return (string)GetProperty("Name"); }
        }

        public IEnumerator<Worksheet> GetEnumerator()
        {
            for (var i = 1; i <= Count; i++) 
                yield return this[i];
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }
    }

    internal class Worksheet : ComWrapper
    {
        internal Worksheet(object worksheet) : base(worksheet) { }

        public string Name
        {
            get { return (string)GetProperty("Name"); }
            set { SetProperty("Name", value); }
        }

        public int Index
        {
            get { return (int)GetProperty("Index"); }
        }

        public Range Cells
        {
            get { return new Range(GetProperty("Cells")); }
        }

        public Range Range(string cell1)
        {
            return new Range(GetProperty("Range", cell1));
        }

        public Range Range(string cell1, string cell2)
        {
            return new Range(GetProperty("Range", cell1, cell2));
        }

        public Range Range(string cell1, Range cell2)
        {
            return new Range(GetProperty("Range", cell1, cell2.ComObject));
        }

        public Range Range(Range cell1, string cell2)
        {
            return new Range(GetProperty("Range", cell1.ComObject, cell2));
        }

        public Range Range(Range cell1, Range cell2)
        {
            return new Range(GetProperty("Range", cell1.ComObject, cell2.ComObject));
        }

        public Shapes Shapes
        {
            get { return new Shapes(GetProperty("Shapes")); }
        }

        public void Activate()
        {
            InvokeMethod("Activate");
        }

        public void Protect(string password)
        {
            InvokeMethod("Protect", password);
        }

        public void Protect(string password, bool drawingObjects)
        {
            InvokeMethod("Protect", password, drawingObjects);
        }

        public void Protect(string password, bool drawingObjects, bool contents)
        {
            InvokeMethod("Protect", password, drawingObjects, contents);
        }

        public void Protect(string password, bool drawingObjects, bool contents, bool scenarios)
        {
            InvokeMethod("Protect", password, drawingObjects, contents, scenarios);
        }

        public void Protect(string password, bool drawingObjects, bool contents, bool scenarios, bool userInterfaceOnly)
        {
            InvokeMethod("Protect", password, drawingObjects, contents, scenarios, userInterfaceOnly);
        }

        public void Protect(string password, bool drawingObjects, bool contents, bool scenarios, bool userInterfaceOnly, bool allowFormattingCells)
        {
            InvokeMethod("Protect", password, drawingObjects, contents, scenarios, userInterfaceOnly, allowFormattingCells);
        }

        public void Protect(string password, bool drawingObjects, bool contents, bool scenarios, bool userInterfaceOnly, bool allowFormattingCells, bool allowFormattingColumns)
        {
            InvokeMethod("Protect", password, drawingObjects, contents, scenarios, userInterfaceOnly, allowFormattingCells, allowFormattingColumns);
        }

        public void Protect(string password, bool drawingObjects, bool contents, bool scenarios, bool userInterfaceOnly, bool allowFormattingCells, bool allowFormattingColumns, bool allowFormattingRows)
        {
            InvokeMethod("Protect", password, drawingObjects, contents, scenarios, userInterfaceOnly, allowFormattingCells, allowFormattingColumns, allowFormattingRows);
        }

        public PageSetup PageSetup
        {
            get { return new PageSetup(GetProperty("PageSetup")); }
        }
    }

    internal enum XlDirection
    {
        xlDown = -4121,
        xlToLeft = -4159,
        xlToRight = -4161,
        xlUp = -4162
    }

    internal class Range : ComWrapper, IEnumerable<Range>
    {
        internal Range(object range) : base(range) { }

        public object Value
        {
            get { return GetProperty("Value"); }
            set { SetProperty("Value", value); }
        }

        public string Formula
        {
            get { return (string)GetProperty("Formula"); }
            set { SetProperty("Formula", value); }
        }

        public string NumberFormat
        {
            get { return (string)GetProperty("NumberFormat"); }
            set { SetProperty("NumberFormat", value); }
        }

        public Range this[int row]
        {
            get { return new Range(GetProperty("Item", row)); }
        }

        public Range this[int row, int column]
        {
            get { return new Range(GetProperty("Item", row, column)); }
        }

        public Range Cells
        {
            get { return new Range(GetProperty("Cells")); }
        }

        public int Count
        {
            get { return (int)GetProperty("Count"); }
        }

        public Range SpecialCells(XlCellType cellType)
        {
            return new Range(InvokeMethod("SpecialCells", cellType));
        }

        public Range SpecialCells(XlCellType cellType, XlSpecialCellsValue value)
        {
            return new Range(InvokeMethod("SpecialCells", cellType, value));
        }

        public string Address()
        {
            return (string)GetProperty("Address");
        }

        public string Address(bool rowAbsolute)
        {
            return (string)GetProperty("Address", rowAbsolute);
        }

        public string Address(bool rowAbsolute, bool columnAbsolute)
        {
            return (string)GetProperty("Address", rowAbsolute, columnAbsolute);
        }

        public int Column
        {
            get { return (int)GetProperty("Column"); }
        }

        public int Row
        {
            get { return (int)GetProperty("Row"); }
        }

        public void Merge()
        {
            InvokeMethod("Merge");
        }

        public object ColumnWidth
        {
            get { return GetProperty("ColumnWidth"); }
            set { SetProperty("ColumnWidth", value); }
        }

        public object RowHeight
        {
            get { return GetProperty("RowHeight"); }
            set { SetProperty("RowHeight", value); }
        }

        public Range Columns
        {
            get { return new Range(GetProperty("Columns")); }
        }

        public Range Rows
        {
            get { return new Range(GetProperty("Rows")); }
        }

        public Font Font
        {
            get { return new Font(GetProperty("Font")); }
        }

        public Interior Interior
        {
            get { return new Interior(GetProperty("Interior")); }
        }

        public Borders Borders
        {
            get { return new Borders(GetProperty("Borders")); }
        }

        public Range EntireRow
        {
            get { return new Range(GetProperty("EntireRow")); }
        }

        public Range EntireColumn
        {
            get { return new Range(GetProperty("EntireColumn")); }
        }

        public object Hidden
        {
            get { return GetProperty("Hidden"); }
            set { SetProperty("Hidden", value); }
        }

        public Range Resize(int rowSize)
        {
            return new Range(GetProperty("Resize", rowSize));
        }

        public Range Resize(int rowSize, int columnSize)
        {
            return new Range(GetProperty("Resize", rowSize, columnSize));
        }

        public Range Offset(int rowOffset)
        {
            return new Range(GetProperty("Offset", rowOffset));
        }

        public Range Offset(int rowOffset, int columnOffset)
        {
            return new Range(GetProperty("Offset", rowOffset, columnOffset));
        }

        public HorizontalAlignment HorizontalAlignment
        {
            get { return (HorizontalAlignment)GetProperty("HorizontalAlignment"); }
            set { SetProperty("HorizontalAlignment", value); }
        }

        public VerticalAlignment VerticalAlignment
        {
            get { return (VerticalAlignment)GetProperty("VerticalAlignment"); }
            set { SetProperty("VerticalAlignment", value); }
        }

        public FormatConditions FormatConditions
        {
            get { return new FormatConditions(GetProperty("FormatConditions")); }
        }

        public object WrapText
        {
            get { return GetProperty("WrapText"); }
            set { SetProperty("WrapText", value); }
        }

        public void Select()
        {
            InvokeMethod("Select");
        }

        public double Left
        {
            get { return (double)GetProperty("Left"); }
        }

        public double Top
        {
            get { return (double)GetProperty("Top"); }
        }

        public double Width
        {
            get { return (double)GetProperty("Width"); }
        }

        public double Height
        {
            get { return (double)GetProperty("Height"); }
        }

        public object Locked
        {
            get { return GetProperty("Locked"); }
            set { SetProperty("Locked", value); }
        }

        public Range End(XlDirection direction)
        {
            return new Range(GetProperty("End", direction));
        }

        public Validation Validation
        {
            get { return new Validation(GetProperty("Validation")); }
        }

        public IEnumerator<Range> GetEnumerator()
        {
            for (var i = 1; i <= Rows.Count; i++)
                for (var j = 1; j <= Columns.Count; j++)
                    yield return Cells[i, j];
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }

        public object Orientation
        {
            get { return GetProperty("Orientation"); }
            set { SetProperty("Orientation", value); }
        }

        public bool Replace(string what, string replacement, XlLookAt lookAt, XlSearchOrder searchOrder, bool matchCase,
            bool matchByte, object searchFormat, object replaceFormat)
        {
            return (bool)InvokeMethod("Replace", what, replacement, lookAt, searchOrder, matchCase, matchByte, searchFormat, replaceFormat);
        }
    }

    internal enum XlLookAt
    {
        xlWhole = 1,
        xlPart = 2,
    }

    internal enum XlSearchOrder
    {
        xlByRows = 1,
        xlByColumns = 2,
    }

    internal class Validation : ComWrapper
    {
        internal Validation(object validation) : base(validation) { }

        public void Add(XlDVType type, XlDVAlertStyle alertStyle, XlFormatConditionOperator @operator, string formula1)
        {
            InvokeMethod("Add", type, alertStyle, @operator, formula1);
        }

        public void Add(XlDVType type, XlDVAlertStyle alertStyle, XlFormatConditionOperator @operator, string formula1, string formula2)
        {
            InvokeMethod("Add", type, alertStyle, @operator, formula1, formula2);
        }

        public string ErrorMessage
        {
            get { return (string)GetProperty("ErrorMessage"); }
            set { SetProperty("ErrorMessage", value); }
        }

        public string ErrorTitle
        {
            get { return (string)GetProperty("ErrorTitle"); }
            set { SetProperty("ErrorTitle", value); }
        }

        public bool ShowError
        {
            get { return (bool)GetProperty("ShowError"); }
            set { SetProperty("ShowError", value); }
        }

        public string InputMessage
        {
            get { return (string)GetProperty("InputMessage"); }
            set { SetProperty("InputMessage", value); }
        }

        public string InputTitle
        {
            get { return (string)GetProperty("InputTitle"); }
            set { SetProperty("InputTitle", value); }
        }

        public bool ShowInput
        {
            get { return (bool)GetProperty("ShowInput"); }
            set { SetProperty("ShowInput", value); }
        }

        public bool IgnoreBlank
        {
            get { return (bool)GetProperty("IgnoreBlank"); }
            set { SetProperty("IgnoreBlank", value); }
        }
    }

    internal enum XlDVType
    {
        xlValidateInputOnly = 0,
        xlValidateWholeNumber = 1,
        xlValidateDecimal = 2,
        xlValidateList = 3,
        xlValidateDate = 4,
        xlValidateTime = 5,
        xlValidateTextLength = 6,
        xlValidateCustom = 7,
    }

    internal enum XlDVAlertStyle
    {
        xlValidAlertStop = 1,
        xlValidAlertWarning = 2,
        xlValidAlertInformation = 3,
    }

    internal enum XlFormatConditionOperator
    {
        xlBetween = 1,
        xlNotBetween = 2,
        xlEqual = 3,
        xlNotEqual = 4,
        xlGreater = 5,
        xlLess = 6,
        xlGreaterEqual = 7,
        xlLessEqual = 8,
    }

    internal class Font : ComWrapper
    {
        internal Font(object font) : base(font) { }

        public string Name
        {
            get { return (string)GetProperty("Name"); }
            set { SetProperty("Name", value); }
        }

        public object Bold
        {
            get { return GetProperty("Bold"); }
            set { SetProperty("Bold", value); }
        }

        public object Underline
        {
            get { return GetProperty("Underline"); }
            set { SetProperty("Underline", value); }
        }

        public object Italic
        {
            get { return GetProperty("Italic"); }
            set { SetProperty("Italic", value); }
        }

        public object Size
        {
            get { return GetProperty("Size"); }
            set { SetProperty("Size", value); }
        }

        public object ColorIndex
        {
            get { return GetProperty("ColorIndex"); }
            set { SetProperty("ColorIndex", value); }
        }

        public object Color
        {
            get { return GetProperty("Color"); }
            set { SetProperty("Color", value); }
        }
    }

    internal class Interior : ComWrapper
    {
        internal Interior(object interior) : base(interior) { }

        public object ColorIndex
        {
            get { return GetProperty("ColorIndex"); }
            set { SetProperty("ColorIndex", value); }
        }

        public object Color
        {
            get { return GetProperty("Color"); }
            set { SetProperty("Color", value); }
        }
    }

    internal class Borders : ComWrapper
    {
        internal Borders(object borders) : base(borders) { }

        public Border this[XlBordersIndex index]
        {
            get { return new Border(GetProperty("Item", index)); }
        }
    }

    internal class Border : ComWrapper
    {
        internal Border(object border) : base(border) { }

        public XlLineStyle LineStyle
        {
            get { return (XlLineStyle)GetProperty("LineStyle"); }
            set { SetProperty("LineStyle", value); }
        }

        public XlBorderWeight Weight
        {
            get { return (XlBorderWeight)GetProperty("Weight"); }
            set { SetProperty("Weight", value); }
        }

        public object ColorIndex
        {
            get { return GetProperty("ColorIndex"); }
            set { SetProperty("ColorIndex", value); }
        }

        public object Color
        {
            get { return GetProperty("Color"); }
            set { SetProperty("Color", value); }
        }
    }

    internal class FormatConditions : ComWrapper
    {
        internal FormatConditions(object formatConditions) : base(formatConditions) { }

        public FormatCondition this[int index]
        {
            get { return new FormatCondition(GetProperty("Item", index)); }
        }

        public void Delete()
        {
            InvokeMethod("Delete");
        }

        public FormatCondition Add(XlFormatConditionType type)
        {
            return new FormatCondition(InvokeMethod("Add", type));
        }

        public FormatCondition Add(XlFormatConditionType type, XlFormatConditionOperator @operator)
        {
            return new FormatCondition(InvokeMethod("Add", type, @operator));
        }

        public FormatCondition Add(XlFormatConditionType type, XlFormatConditionOperator @operator, string formula1)
        {
            return new FormatCondition(InvokeMethod("Add", type, @operator, formula1));
        }

        public FormatCondition Add(XlFormatConditionType type, XlFormatConditionOperator @operator, string formula1, string formula2)
        {
            return new FormatCondition(InvokeMethod("Add", type, @operator, formula1, formula2));
        }
    }

    internal class FormatCondition : ComWrapper
    {
        internal FormatCondition(object formatCondition) : base(formatCondition) { }

        public Font Font
        {
            get { return new Font(GetProperty("Font")); }
        }

        public Interior Interior
        {
            get { return new Interior(GetProperty("Interior")); }
        }
    }

    internal class Shapes : ComWrapper
    {
        internal Shapes(object shapes) : base(shapes) { }

        public Shape AddLine(double beginX, double beginY, double endX, double endY)
        {
            return new Shape(InvokeMethod("AddLine", beginX, beginY, endX, endY));
        }

        public Shape AddPicture(string filename, MsoTriState linkToFile, MsoTriState saveWithDocument, float left, float top, float width, float height)
        {
            return new Shape(InvokeMethod("AddPicture", filename, linkToFile, saveWithDocument, left, top, width, height));
        }
    }

    internal class Shape : ComWrapper
    {
        internal Shape(object shape) : base(shape) { }

        public LineFormat Line
        {
            get { return new LineFormat(GetProperty("Line")); }
        }
    }

    internal class LineFormat : ComWrapper
    {
        internal LineFormat(object lineFormat) : base(lineFormat) { }

        public MsoArrowheadStyle EndArrowheadStyle
        {
            get { return (MsoArrowheadStyle)GetProperty("EndArrowheadStyle"); }
            set { SetProperty("EndArrowheadStyle", value); }
        }

        public MsoArrowheadLength EndArrowheadLength
        {
            get { return (MsoArrowheadLength)GetProperty("EndArrowheadLength"); }
            set { SetProperty("EndArrowheadLength", value); }
        }

        public MsoArrowheadWidth EndArrowheadWidth
        {
            get { return (MsoArrowheadWidth)GetProperty("EndArrowheadWidth"); }
            set { SetProperty("EndArrowheadWidth", value); }
        }

        public ColorFormat BackColor
        {
            get { return new ColorFormat(GetProperty("BackColor")); }
        }

        public ColorFormat ForeColor
        {
            get { return new ColorFormat(GetProperty("ForeColor")); }
        }
    }

    internal class ColorFormat : ComWrapper
    {
        internal ColorFormat(object colorFormat) : base(colorFormat) { }

        public long SchemeColor
        {
            get { return (long)GetProperty("SchemeColor"); }
            set { SetProperty("SchemeColor", value); }
        }
    }

    internal class Window : ComWrapper
    {
        public Window(object window) : base(window) { }

        public bool DisplayGridLines
        {
            get { return (bool)GetProperty("DisplayGridLines"); }
            set { SetProperty("DisplayGridLines", value); }
        }

        public bool FreezePanes
        {
            get { return (bool)GetProperty("FreezePanes"); }
            set { SetProperty("FreezePanes", value); }
        }
    }

    internal class PageSetup : ComWrapper
    {
        internal PageSetup(object pageSetup) : base(pageSetup) { }

        public string PrintArea
        {
            get { return (string)GetProperty("PrintArea"); }
            set { SetProperty("PrintArea", value); }
        }

        public object FitToPagesTall
        {
            get { return GetProperty("FitToPagesTall"); }
            set { SetProperty("FitToPagesTall", value); }
        }

        public object FitToPagesWide
        {
            get { return GetProperty("FitToPagesWide"); }
            set { SetProperty("FitToPagesWide", value); }
        }

        public object Zoom
        {
            get { return GetProperty("Zoom"); }
            set { SetProperty("Zoom", value); }
        }

        public XlPageOrientation Orientation
        {
            get { return (XlPageOrientation)GetProperty("Orientation"); }
            set { SetProperty("Orientation", value); }
        }

        public double LeftMargin
        {
            get { return (double)GetProperty("LeftMargin"); }
            set { SetProperty("LeftMargin", value); }
        }

        public double RightMargin
        {
            get { return (double)GetProperty("RightMargin"); }
            set { SetProperty("RightMargin", value); }
        }

        public double TopMargin
        {
            get { return (double)GetProperty("TopMargin"); }
            set { SetProperty("TopMargin", value); }
        }

        public double BottomMargin
        {
            get { return (double)GetProperty("BottomMargin"); }
            set { SetProperty("BottomMargin", value); }
        }

        public double HeaderMargin
        {
            get { return (double)GetProperty("HeaderMargin"); }
            set { SetProperty("HeaderMargin", value); }
        }

        public double FooterMargin
        {
            get { return (double)GetProperty("FooterMargin"); }
            set { SetProperty("FooterMargin", value); }
        }
    }

    internal enum XlPageOrientation
    {
        xlPortrait = 1,
        xlLandscape = 2,
    }

    internal enum MsoTriState
    {
        msoCTrue = 1,
        msoFalse = 0,
        msoTriStateMixed = -2,
        msoTriStateToggle = -3,
        msoTrue = -1
    }

    internal enum XlCalculation : int
    {
        xlAutomatic = -4105,
        xlManual = -4135,
        xlSemiautomatic = 2
    }

    internal enum XlUpdateLinks
    {
        xlUpdateLinksUserSetting = 1,
        xlUpdateLinksNever = 2,
        xlUpdateLinksAlways = 3
    }

    internal enum XlFileAccess
    {
        xlReadWrite = 2,
        xlReadOnly = 3
    }

    internal enum HorizontalAlignment
    {
        xlLeft = -4131,
        xlCenter = -4108,
        xlRight = -4152,
    }

    internal enum VerticalAlignment
    {
        xlTop = -4160,
        xlCenter = -4108,
        xlBottom = -4107,
    }

    internal enum XlCellType
    {
        xlCellTypeAllFormatConditions = -4172,
        xlCellTypeAllValidation = -4174,
        xlCellTypeBlanks = 4,
        xlCellTypeComments = -4144,
        xlCellTypeConstants = 2,
        xlCellTypeFormulas = -4123,
        xlCellTypeLastCell = 11,
        xlCellTypeSameFormatConditions = -4173,
        xlCellTypeSameValidation = -4175,
        xlCellTypeVisible = 12
    }

    internal enum XlSpecialCellsValue
    {
        xlErrors = 16,
        xlLogical = 4,
        xlNumbers = 1,
        xlTextValues = 2,
    }

    internal enum XlBordersIndex
    {
        xlDiagonalDown = 5,
        xlDiagonalUp = 6,
        xlEdgeBottom = 9,
        xlEdgeLeft = 7,
        xlEdgeRight = 10,
        xlEdgeTop = 8,
        xlInsideHorizontal = 12,
        xlInsideVertical = 11
    }

    internal enum XlLineStyle
    {
        xlContinuous = 1,
        xlDash = -4115,
        xlDashDot = 4,
        xlDashDotDot = 5,
        xlDot = -4118,
        xlDouble = -4119,
        xlLineStyleNone = -4142,
        xlSlantDashDot = 13
    }

    internal enum XlBorderWeight
    {
        xlHairline = 1,
        xlMedium = -4138,
        xlThick = 4,
        xlThin = 2
    }

    internal enum XlFormatConditionType
    {
        xlCellValue = 1,
        xlExpression = 2,
    }

    internal enum MsoArrowheadStyle
    {
        msoArrowheadTriangle = 2,
    }

    internal enum MsoArrowheadLength
    {
        mosArrowheadLengthMedium = 2,
    }

    internal enum MsoArrowheadWidth
    {
        MsoArrowheadWidthMedium = 2,
    }
}
